@@ -6,4 +6,6 @@ import synthdata
 import cluster
 import pica
 import atrous
+import mmt
+import multiscale
 import mvm
