@@ -46,7 +46,13 @@ def xcorrdist(v1,v2):
     return 1/np.correlate(v1,v2)
 
 
-## TODO: distance_matrix
+## TODO:
+### [X] distance_matrix
+### [ ] sparse distance matrix
+### [ ] rename dist_fn to metrics
+### [ ] use scipy.spatial for distance calculations
+
+
 
 ##---------------------------
 
@@ -226,7 +232,7 @@ def quality_threshold(points, max_diam, dist_fn = euclidean):
 ## note: can't work with points as ndarrays because
 ## 1. they are not hashable (convert to tuples) and
 ## 2. can't be compared simply by p1 == p2.
-def dbscan(points, eps, min_pts, dist_fn = euclidean,
+def dbscan_old(points, eps, min_pts, dist_fn = euclidean,
            verbose=False):
     """ Implementation of DBSCAN density-based clustering algorithm
 
@@ -281,55 +287,74 @@ def dbscan(points, eps, min_pts, dist_fn = euclidean,
     clusters.sort(key=lambda x: len(x), reverse = True)
     return [Cluster(c, dist_fn = dist_fn) for c in clusters]
 
-#expandCluster(P, N, C, eps, MinPts)
-#   add P to cluster C
-#   for each point P' in N 
-#      if P' is not visited
-#         mark P' as visited
-#         N' = getNeighbors(P', eps)
-#         if sizeof(N') >= MinPts
-#            N = N joined with N'
-#      if P' is not yet member of any cluster
-#         add P' to cluster C
+## here I used dbscan_.py from sklearn as a reference
+def dbscan(points, eps, min_pts, distances=None,dist_fn='euclidean',verbose=True):
+    """ Implementation of DBSCAN density-based clustering algorithm
 
-### here i tried with pre-calculation of distances between points
-### for dbscan. Didn't help the speed though
-def _distance_dict_redun(points, dist_fn=euclidean):
-    "redundant mapping of pairwise distances"
-    dd = {}
-    for p in points: dd[p] = {}
-    for p1,p2 in itt.permutations(points,2):
-        dd[p1][dist_fn(p1,p2)] = p2
-    return dd
+    Parameters:
+      - `points`: input collection of points. points must be hashable
+                  (i.e. a point is a tuple of coordinates)
+      - `eps`: (`number`) --  neighborhood radius
+      - `min_pts`: (`number`) -- minimal number of neighborhood points
+      - `dist_fn`: (`function`) -- distance measure, [``euclidean``]
+      - `verbose`: (`bool`) -- if ``True``, be verbose
 
-def _kdist(points, k, dist_fn = euclidean):
-    dd = _distance_dict_redun(points, dist_fn)
-    dists = []
-    for point in points:
-        x = sorted(dd[point].keys())
-        dists.append(x[k])
-    return sorted(dists, reverse=True)
+    Returns:
+      a list of clusters (each cluster is a Cluster class instance)
+    """
+    points = np.asarray(points)
+    L = len(points)
+    if distances is None:
+	D = _pairwise_euclidean_distances(points)
+    else:
+	D = distances
+    #neighborhoods = [np.where(x <= eps)[0] for x in D]
+    neighborhoods = [np.where(x <= eps)[0] for x in D]
+    labels = -np.ones(L) # everything is noise at start
+    core_points = []
+    label_marker = 0
+    perm = np.random.permutation(L) # we iterate through points in random order
+    for k in perm: 
+	if labels[k] != -1 or len(neighborhoods[k]) < min_pts:
+	    # either visited or not a core point
+	    continue
+	core_points.append(k)
+	labels[k] = label_marker
+	# expand cluster
+	candidates = [k]
+	while len(candidates) > 0:
+	    new_candidates = []
+	    for c in candidates:
+		noise = np.where(labels[neighborhoods[c]] == -1)[0]
+		noise = neighborhoods[c][noise] # ?
+		labels[noise] = label_marker
+		for neigh in noise:
+		    if len(neighborhoods[neigh]) >= min_pts:
+			new_candidates.append(neigh) # another core point
+			core_points.append(neigh)
+	    candidates = new_candidates
+	label_marker += 1
+    clusters = sorted([points[labels==lm] for lm in range(label_marker)],
+		      key=lambda x: len(x), reverse=True)
+    clusters = [Cluster(c, dist_fn = dist_fn) for c in clusters]
+    
+    return clusters, core_points, labels
 
-def _distance_dict(points, dist_fn = euclidean):
-    "non-redundant mapping of distances"
-    dd = {}
-    for pair in itt.combinations(points,2):
-        dd[pair] = dist_fn(*pair)
-    return dd
 
-def _alldistances_test1(points, dist_fn = euclidean):
-    for pair in itt.combinations(points,2):
-        dist_fn(*pair)
+def _pairwise_euclidean_distances(points):
+    """pairwise euclidean distances between points.
+    Calculated as distance between vectors x and y:
+    d = sqrt(dot(x,x) -2*dot(x,y) + dot(Y,Y))
+    """
+    X = np.asarray(points)
+    XX = np.sum(X*X, axis=1)[:,np.newaxis]
+    D = -2 * np.dot(X,X.T) + XX + XX.T
+    np.maximum(D, 0, D)
+    # todo triangular matrix, sparse matrix
+    return np.sqrt(D)
+    
+
         
-def _neighbours_dict(points, eps,dist_fn = euclidean):
-    nd = {}
-    #dd = distance_dict(points,dist_fn)
-    for p in points: nd[p] = []
-    for p1,p2 in itt.permutations(points,2):
-        d =  dist_fn(p1,p2)
-        if d <  eps:
-            nd[p1].append(p2)
-    return nd
 ###-----------------------------------
 
 
@@ -474,8 +499,18 @@ def locations(shape):
     """ all locations for a shape; substitutes nested cycles
     """
     return itt.product(*map(xrange, shape))
-    
+
 def mask2points(mask):
+    "mask to a list of points, as row,col"
+    points = []
+    for loc in locations(mask.shape):
+        if mask[loc]:
+            points.append(loc) 
+    return points
+
+    
+def mask2pointsr(mask):
+    "mask to a list of points, with X,Y coordinates reversed"
     points = []
     for loc in locations(mask.shape):
         if mask[loc]:
@@ -492,35 +527,39 @@ def cluster2array(c):
 def points2array(points,dtype=np.float64):
     return np.array(points, dtype=dtype)
 
-def surfconvert(frame):
+
+def surfconvert(frame, mask):
+    from imfun import lib
     out = []
     nr,nc = map(float, frame.shape)
+    space_scale = max(nr, nc)
     f = lib.rescale(frame)
     for r in range(int(nr)):
         for c in range(int(nc)):
-            out.append([c/nc,r/nr, f[r,c]])
+	    if not mask[r,c]:
+		out.append([c/space_scale,r/space_scale, f[r,c]])
     return np.array(out)
 
 
 
-###### Now, use mdp (no good so far)
-try:
-    import mdp
-    def do_gng(arr,**kwargs):
-        gng = mdp.nodes.GrowingNeuralGasNode(**kwargs)
-        gng.train(arr)
-        gng.stop_training()
-        return gng
+## ###### Now, use mdp (no good so far)
+## try:
+##     import mdp
+##     def do_gng(arr,**kwargs):
+##         gng = mdp.nodes.GrowingNeuralGasNode(**kwargs)
+##         gng.train(arr)
+##         gng.stop_training()
+##         return gng
 
 
-    def plot_mdp_nodes(arr, gng):
-        #figure(figsize=(6,6));
-        #scatter(*arr.T, color='k', s = 0.3)
-        colors = ['r', 'b','g','c','m','y']
-        objs = gng.graph.connected_components()
-        for j,o in enumerate(objs):
-            coords = np.array([node.data.pos for node in o])
-            scatter(coords[:,1],coords[:,2],color=colors[j%len(colors)])
-except:
-    pass
+##     def plot_mdp_nodes(arr, gng):
+##         #figure(figsize=(6,6));
+##         #scatter(*arr.T, color='k', s = 0.3)
+##         colors = ['r', 'b','g','c','m','y']
+##         objs = gng.graph.connected_components()
+##         for j,o in enumerate(objs):
+##             coords = np.array([node.data.pos for node in o])
+##             scatter(coords[:,1],coords[:,2],color=colors[j%len(colors)])
+## except:
+##     pass
         
