@@ -29,7 +29,7 @@ def iter_files(pattern, loadfn):
     return itt.imap(loadfn, sorted_file_names(pattern))
 
 
-def img_getter(frame, ch, pngflag):
+def img_getter(frame, ch):
     """A wrapper to extract color channel from image files.
     :returns: 2D matrix with intensity values
     """
@@ -37,23 +37,8 @@ def img_getter(frame, ch, pngflag):
         out = frame[:,:,ch]
     else:
         out = frame
-    if pngflag:
-        out = out[::-1,:]
     return out
 
-def _fseq_from_glob_deprecated(pattern, ch=None, loadfn=np.load):
-    """Sequence of frames from filenames matching a glob
-    
-    :param pattern:  glob-pattern for file names
-    :param ch: color channel
-    :param loadfn : [np.load], function to load data from a file by its name
-
-    :returns: iterator over 2D matrices
-    """
-    pngflag = (pattern[-3:] == 'png')
-    return itt.imap(lambda frame: img_getter(frame, ch, pngflag),
-                    iter_files(pattern, loadfn))
-
 
 def fseq_from_glob(pattern, ch=None, loadfn=np.load):
     """Return sequence of frames from filenames matching a glob.
@@ -66,9 +51,8 @@ def fseq_from_glob(pattern, ch=None, loadfn=np.load):
     Returns:
       - iterator over frames. `2D` if `ch` is `int`, `3D` if ch is `None`
     """
-    pngflag = (pattern[-3:] == 'png')
     if ch is not None:
-	return itt.imap(lambda frame: img_getter(frame, ch, pngflag),
+	return itt.imap(lambda frame: img_getter(frame, ch),
 			iter_files(pattern, loadfn))
     else:
 	return iter_files(pattern, loadfn)
@@ -100,6 +84,10 @@ class FrameSequence(object):
         else:
             self.dx, self.dy = dx,dy
 
+    def pipeline(self):
+	"""Return the composite function to process frames based on self.fns"""
+	return lib.flcompose(identity, *self.fns)
+
     def std(self):
 	"""get standard deviation of the data"""
         a = self.as3darray()
@@ -107,13 +95,12 @@ class FrameSequence(object):
 
     def data_range(self):
         """Return global range (`min`, `max`) values for the sequence"""
-        ranges = np.array([(np.min(f), np.max(f)) for f in self.frames()])
-        minv,maxv = np.min(ranges[:,0]), np.max(ranges[:,1])
-        return (minv, maxv)
-
-    def pipeline(self):
-	"""Return the composite function to process frames based on self.fns"""
-	return lib.flcompose(identity, *self.fns)
+        # need this strange syntax for min/max for multi-channel images
+        rfn = lambda fn: lambda x: fn(fn(x, axis=0),axis=0)
+        ranges = np.array([(rfn(np.min)(f), rfn(np.max)(f)) for f in self.frames()])
+        minv,maxv = np.min(ranges[:,0],axis=0), np.max(ranges[:,1],axis=0)
+        return np.array([minv,maxv]).T
+        #return (minv, maxv)
 
     def data_percentile(self, p):
 	"""Return a percentile `p` value on data.
@@ -123,7 +110,12 @@ class FrameSequence(object):
 	     Percentile to compute which must be between 0 and 100 inclusive.
 	  
 	"""
-        return  np.percentile(self.as3darray(),p)
+        sh = self.shape()
+        arr = self.as3darray()
+        if len(sh) == 2:
+            return  np.percentile(arr,p)
+        else:
+            return [np.percentile(arr[...,k],p) for k in range(sh[2])]
 
     def timevec(self,):
         """Return a vector of time stamps, calculated based on `self.dt`"""
@@ -133,18 +125,27 @@ class FrameSequence(object):
     def mask_reduce(self, mask):
         """Return `1D` vector from a mask (or slice), taking average value within
         this mask in each frame"""
-        return np.asarray([np.mean(f[mask]) for f in self.frames()])
+        return np.asarray([np.mean(f[mask],axis=0) for f in self.frames()])
+    def softmask_reduce(self,mask):
+	"""Return `1D` vector from a mask (or slice), taking average value within
+        this mask in each frame, weighted by mask values between 0 and 1"""
+        sh = self.shape()
+        mask2d = mask
+        if len(sh) >2 :
+            mask = np.dstack([mask]*sh[-1])
+        return np.asarray([np.mean((f*mask)[mask2d>0],axis=0) for f in self.frames()])
 
-    def frame_slices(self, sliceobj):
-        """Return iterator over subframes (slices defined by `sliceobj` parameter).
-	When `sliceobj` is `None`, full frames are returned 
+
+    def frame_slices(self, crop):
+        """Return iterator over subframes (slices defined by `crop` parameter).
+	When `crop` is `None`, full frames are returned 
 	"""
-        if sliceobj:
-            return (f[sliceobj] for f in self.frames())
+        if crop:
+            return (f[crop] for f in self.frames())
         else:
             return self.frames()
 
-    def time_project(self,start=None,stop=None,fn=np.mean):
+    def time_project(self,fslice=None,fn=np.mean,crop=None):
 	"""Apply an ``f(vector) -> scalar`` function for each pixel.
 
 	This is a more general (and often faster) function than
@@ -152,103 +153,105 @@ class FrameSequence(object):
 	memory as it makes a call to self.as3darray, while the other two don't
 	
 	Parameters:
-	  - `start`: (`int` or `None`) -- frame to start at
-	  - `stop`: (`int` or `None`) -- frame to stop at
+	  - `fslice`: (`int`, `slice` or `None`) -- go throug these frames
 	  - `fn` (`func`) -- function to apply (`np.mean` by default)
 
         Returns:
 	  - `2D` `array` -- a projected frame
 	"""
-	L = self.length()
-	start = ifnot(start,0)
-        stop = min(ifnot(stop,L), L)
-	out = np.zeros(self.shape())
-	for v,r,c in self.pix_iter():
-	    out[r,c] = fn(v[start:stop])
+        sh = self.shape(crop)
+	out = np.zeros(sh)
+        if len(sh)>2:
+            fn = lambda a: np.mean(a, axis=0)
+	for v,r,c in self.pix_iter(fslice=fslice,crop=crop):
+	    out[r,c] = fn(v)
 	return out
 
-    def mean_frame(self, start=None, stop=None):
+    def mean_frame(self, fslice=None):
         """Return average image over N frames starting.
 
 	Starts at `start`, stops at `stop`. A value of `None`
 	corresponds to 0 for `start` and last frame for `stop`
 	"""
-        L = self.length()
-        frameit = itt.imap(_dtype_, self.frames())
+        if fslice is None or type(fslice) is int:
+            fslice = (fslice, )
+        frameit = itt.imap(_dtype_, itt.islice(self.frames(), *fslice))
         res = np.copy(frameit.next())
-	if start is None: start =0
-        stop = min(ifnot(stop,L), L)
-	count = 1
+        count = 0
         for k,frame in enumerate(frameit):
-	    if k < start: continue
-	    elif k>=stop: break
             res += frame
 	    count += 1
         return res/(count)
     
-    def max_project(self, start = None, stop=None):
+    def max_project(self, fslice):
 	"""Return max-projection image from start to stop frame
 	(all by	default)
 	"""
-        L = self.length()
-        frameit = itt.imap(_dtype_, self.frames())
-	start = max(ifnot(start, 0), 0)
-        stop = min(ifnot(stop,L), L)
+        if fslice is None or type(fslice) is int:
+            fslice = (fslice, )
 	out = frameit.next()
+        frameit = itt.imap(_dtype_, itt.islice(self.frames(), *fslice))
         for k,frame in enumerate(frameit):
 	    if k <= start: continue
 	    elif k>stop: break
 	    out = np.max([out, frame], axis=0)
         return out
 
-    def aslist(self, maxN=None, sliceobj=None):
-        """Return the frames as a list up to `maxN` frames
-	taking ``f[sliceobj]`` slices.
+    def aslist(self, max_frames=None, crop=None):
+        """Return the frames as a list up to `max_frames` frames
+	taking ``f[crop]`` slices.
 	"""
-        return list(self.asiter(maxN,sliceobj))
+        print "fseq.aslist is deprecated and will be removed\
+        use the  __getitem__ interface, e.g. fseq[10:20]"
+        return list(self.asiter(max_frames,crop))
 
-    def asiter(self, maxN=None, sliceobj=None):
-        """Return an iterator over the up to `maxN` frames taking
-	``f[sliceobj]`` slices
+    def asiter(self, fslice=None, crop=None):
+        """Return an iterator over the frames taking frames from fslice and
+	``f[crop]`` for each frame 
 	"""
-        fiter = self.frame_slices(sliceobj)
-        return itt.islice(fiter, maxN)
+        fiter = self.frame_slices(crop)
+        if type(fslice) is int :
+            fslice = (fslice, )
+        return itt.islice(fiter, *fslice)
 
-    def as3darray(self,  maxN=None,sliceobj=None,
+    def as3darray(self,  fslice=None, crop=None,
                   dtype = _dtype_):
 	"""Return the frames as a `3D` array.
 
-	An alternative way is to use the __get_item__ interface:
-	``data = np.asarray(fs[10:100])``
+	//An alternative way is to use the __get_item__ interface:
+	//``data = np.asarray(fs[10:100])``
 	
 
 	Parameters:
-	  - `maxN`: (`int` or `None`) -- up to this frame. Up to last frame if
-	    `None`
-	  - `sliceobj`: (`slice` or `None`) -- a slice to take from each frame
+	  - `fslice`: (`int`, `slice` or `None`) -- slice to go through frames
+	  - `crop`: (`slice` or `None`) -- a crop (tuple of slices) to take from each frame
 	  - `dtype`: (`type`) -- data type to use. Default, ``np.float64``
 
 	Returns:
 	  `3D` array `d`, where frames are stored in higher dimensions, such
 	  that ``d[0]`` is the first frame, etc.
 	"""
-        fiter = self.frame_slices(sliceobj)
-        shape = self.shape(sliceobj)
-	newshape = [self.length()] + list(shape)
+        if fslice is None or type(fslice) is int:
+            fslice = (fslice, )
+        shape = self.shape(crop)
+	newshape = (self.length(),) + shape
         out = lib.memsafe_arr(newshape, dtype)
-        for k,frame in enumerate(itt.islice(fiter, maxN)):
-            out[k,:,:] = frame
+        for k,frame in enumerate(itt.islice(self.frames(), *fslice)):
+            out[k,:,:] = frame[crop]
+        out = out[:k+1]
         if hasattr (out, 'flush'):
             out.flush()
         return out
     
-    def pix_iter(self, mask=None, maxN=None, rand=False, **kwargs):
+    def pix_iter(self, pmask=None, fslice=None, rand=False,
+		 crop = None,
+		 **kwargs):
         """Return iterator over time signals from each pixel.
 
 	Parameters:
 	  - `mask`: (2D `Bool` array or `None`) -- skip pixels where `mask` is
             `False` if `mask` is `None`, take all pixels
-	  - `maxN`: (`int` or `None`) -- use frames up to `maxN`
+          - `fslice`: (`int`, `slice` or `None`) -- slice to go through frames
 	  - `rand`: (`Bool`) -- whether to go through pixels in a random order
 	  - `**kwargs`: keyword arguments to be passed to `self.as3darray`
 
@@ -256,19 +259,29 @@ class FrameSequence(object):
 	 tuples of `(v,row,col)`, where `v` is the time-series in a pixel at `row,col`
 	
 	"""
-        arr = self.as3darray(maxN, **kwargs)
-        if mask== None:
-            mask = np.ones(self.shape(), np.bool)
-        nrows, ncols = arr.shape[1:]
+        arr = self.as3darray(fslice, crop=crop,**kwargs)
+	sh = self.shape(crop)
+        if pmask== None:
+            pmask = np.ones(sh[:2], np.bool)
+        nrows, ncols = sh[:2]
         rcpairs = [(r,c) for r in xrange(nrows) for c in xrange(ncols)]
         if rand: rcpairs = np.random.permutation(rcpairs)
+	if crop is None:
+	    submask = pmask
+	    r0,c0 = 0,0
+	else:
+	    submask = pmask[crop]
+	    r0,c0 = crop[0].start,crop[1].start
         for row,col in rcpairs:
-            if mask[row,col]:
+	    if r0+row>=sh[0] or c0+col>=sh[1]:
+		continue
+            if submask[row,col]:
                 ## asarray to convert from memory-mapped array
                 yield np.asarray(arr[:,row,col]), row, col
         if hasattr(arr, 'flush'):
             arr.flush()
         del arr
+	return
         
 
     def length(self):
@@ -282,9 +295,9 @@ class FrameSequence(object):
         else:
             return self._length
 
-    def shape(self, sliceobj=None):
+    def shape(self, crop=None):
         "Return the shape of frames in the sequence"
-        return self.frame_slices(sliceobj).next().shape
+        return self.frame_slices(crop).next().shape
 
     def _norm_mavg(self, tau=90., **kwargs):
 	"Return normalized frame sequence"
@@ -313,7 +326,7 @@ class FrameSequence(object):
             the pixels
 	  - `**kwargs``: keyword arguments to be passed to `self.pix_iter`
 	"""
-	nrows, ncols = self.shape()[:2]
+	#nrows, ncols = self.shape()[:2]
 	if kwargs.has_key('dtype'):
 	    dtype = kwargs['dtype']
 	else:
@@ -321,7 +334,7 @@ class FrameSequence(object):
 	L = len(pwfn(np.random.randn(self.length())))
 	#testv = pwfn(self.pix_iter(rand=True,**kwargs).next()[0])
 	#L = len(testv)
-	out = lib.memsafe_arr((L, nrows, ncols), dtype)
+	out = lib.memsafe_arr((L,) + self.shape(), dtype)
         for v, row, col in self.pix_iter(**kwargs):
 	    if verbose:
 		sys.stderr.write('\rworking on pixel (%03d,%03d)'%(row, col))
@@ -387,8 +400,13 @@ class FrameSequence(object):
             sys.stderr.write('\r saving frame %06d of %06d'%(i+1, L))
         plt.close()
 	return fnames
-    def export_mpeg(self, mpeg_name, fps = None, **kwargs):
-	"""Create an mpg  movie from the frame sequence using mencoder.
+    
+    def export_movie_anim(self, mpeg_name, fps=None, start=0, stop=None,
+                          show_title=True, fig_size=(4,4),
+                          vmin=None, vmax=None,**kwargs):
+        """
+        Create an mpg  movie from the frame sequence using mencoder.
+        and mpl.Animation
 
 	Parameters:
 	  - `mpeg_name`: (`str`) -- a name (without extension) for the movie to
@@ -396,20 +414,46 @@ class FrameSequence(object):
 	  - `fps`: (`number`) -- frames per second. If None, use 10/self.dt
 	  - `**kwargs` : keyword arguments to be passed to `self.export_png`
 	"""
-        print "Saving frames as png"
+        from matplotlib import animation
+        import matplotlib.pyplot as plt
+
         if fps is None:
-            fps = 10/self.dt
-	if not kwargs.has_key('path'):
-	    kwargs['path'] = './'
-	if not kwargs.has_key('base'):
-	    kwargs['base'] = '-mpeg-export'
-	path,base = kwargs['path'], kwargs['base']
-	fnames = self.export_img(**kwargs)
-        print 'Running mencoder, this can take a while'
-        mencoder_string = """mencoder mf://%s*.png -mf type=png:fps=%d\
-        -ovc lavc -lavcopts vcodec=wmv2 -oac copy -o %s.mpg"""%(path+base,fps,mpeg_name)
-        os.system(mencoder_string)
-        map(os.remove, fnames)
+            fps = 0.5/self.dt
+
+        if stop is None or stop == -1:
+            stop = self.length()
+	if hasattr(self, 'data'):
+	    vmin = ifnot(vmin, self.data_percentile(1)) # for scale
+	    vmax = ifnot(vmax, self.data_percentile(99)) # for scale
+	else:
+	    vmin = ifnot(vmin, np.min(map(np.min, self.frames())))
+	    vmax = ifnot(vmax, np.min(map(np.max, self.frames())))
+        kwargs.update({'vmin':vmin, 'vmax':vmax})
+        L = min(stop-start, self.length())
+
+        fig = plt.figure(figsize=fig_size)
+        ax = fig.add_subplot(111)
+        plh = ax.imshow(self[start], 
+                        aspect='equal', **kwargs)
+        mytitle = ax.set_title('')
+        def _init():
+            k = 0
+            if show_title:
+                mytitle.set_text('frame: %04d, time: %0.3f s'%(k, k*self.dt))
+            plh.set_data(self[k])
+            return plh, 
+        def _animate(framecount):
+            k = framecount+start
+            plh.set_data(self[k])
+            if show_title:
+                mytitle.set_text('frame: %04d, time: %0.3f s'%(k, k*self.dt))
+            return plh,
+        
+        anim = animation.FuncAnimation(fig, _animate, init_func=_init, frames=L, blit=True)
+        mencoder_extra_args=['-ovc', 'lavc', '-lavcopts', 'vcodec=mpeg4']
+        plt.close(anim._fig)
+        anim.save(mpeg_name, writer='mencoder', fps=fps, extra_args=mencoder_extra_args)
+        return 
 
 class FSeq_arr(FrameSequence):
     """A FrameSequence class as a wrapper around a `3D` array
@@ -434,8 +478,33 @@ class FSeq_arr(FrameSequence):
 	    out[j] = self.pipeline()(x)
 	return out
     def data_percentile(self, p):
-        #from scipy.stats import scoreatpercentile
-        return np.percentile(np.ravel(self.data),p)
+        sh = self.shape()
+        arr = self.data
+        if len(sh) == 2:
+            return  np.percentile(arr,p)
+        else:
+            return [np.percentile(arr[...,k],p) for k in range(sh[2])]
+
+    def pix_iter(self, pmask=None,fslice=None,rand=False,**kwargs):
+	"Iterator over time signals from each pixel (FSeq_arr)"
+	if self.fns == []:
+	    if pmask == None:
+                pmask = np.ones(self.shape()[:2], np.bool)
+	    nrows, ncols = self.shape()[:2]
+	    if kwargs.has_key('dtype'):
+		dtype = kwargs['dtype']
+	    else: dtype=_dtype_
+	    rcpairs = [(r,c) for r in xrange(nrows) for c in xrange(ncols)]
+	    if rand: rcpairs = np.random.permutation(rcpairs)
+	    for row,col in rcpairs:
+		if pmask[row,col]:
+		    v = self.data[:,row,col].copy()
+		    yield np.asarray(v, dtype=dtype), row, col
+	else:
+	    x = super(FSeq_arr,self)
+	    for a in x.pix_iter(pmask=pmask,fslice=fslice,
+				rand=rand,**kwargs):
+		yield a
 
     def frames(self):
 	"""
@@ -480,19 +549,18 @@ class FSeq_glob(FrameSequence):
         return itt.imap(fn, fseq_from_glob(self.pattern,
 					   ifnot(ch, self.ch), self.loadfn))
     def __getitem__(self, val):
-	pngflag = self.file_names[0][-3:] == 'png'
 	fn = self.pipeline()
 	if type(val) is int:
 	    frame = self.loadfn(self.file_names[val])
 	    if self.ch is not None:
-		frame = img_getter(frame, self.ch, pngflag)
+		frame = img_getter(frame, self.ch)
 	    return fn(frame)
 	else:
 	    seq =  map(self.loadfn, self.file_names[val])
 	    if self.ch is not None:
-		seq = (img_getter(f, self.ch, pngflag) for f in seq)
-	    return map(fn, seq)
-	
+		seq = (img_getter(f, self.ch) for f in seq)
+            return map(fn, seq)
+            
 class FSeq_img(FSeq_glob):
     """FrameSequence around a set of image files"""
     loadfn = lambda self,y: imread(y)
@@ -501,6 +569,7 @@ class FSeq_txt(FSeq_glob):
     """FrameSequence around a set of text-image files"""
     loadfn= lambda self,y: np.loadtxt(y)
 
+## TODO: but npy can be just one array
 class FSeq_npy(FSeq_glob):
     """FrameSequence around a set of npy files"""
     loadfn= lambda self,y: np.load(y)
@@ -561,11 +630,11 @@ class FSeq_mlf(FrameSequence):
 	    return itt.imap(fn, itt.imap(self.mlfimg.read_frame, indices[val]))
     def length(self):
         return self.mlfimg.nframes
-    def pix_iter(self, mask=None, maxN=None, rand=False, **kwargs):
-        "Iterator over time signals from each pixel"
+    def pix_iter(self, pmask=None, fslice=None, rand=False, **kwargs):
+        "Iterator over time signals from each pixel, where pmask[pixel] is True"
 	if self.fns == []:
-	    if mask== None:
-		mask = np.ones(self.shape(), np.bool)
+	    if pmask == None:
+		pmask = np.ones(self.shape(), np.bool)
 	    nrows, ncols = self.shape()
 	    if kwargs.has_key('dtype'):
 		dtype = kwargs['dtype']
@@ -573,16 +642,16 @@ class FSeq_mlf(FrameSequence):
 	    rcpairs = [(r,c) for r in xrange(nrows) for c in xrange(ncols)]
 	    if rand: rcpairs = np.random.permutation(rcpairs)
 	    for row,col in rcpairs:
-		if mask[row,col]:
-		    v = [f[row,col] for f in self[:maxN]]
+		if pmask[row,col]:
+		    v = self.mlfimg.read_timeslice((row,col))
 		    yield np.asarray(v, dtype=dtype), row, col
 	else:
 	    x = super(FSeq_mlf,self)
-	    for a in x.pix_iter(mask=mask,maxN=maxN,
+	    for a in x.pix_iter(pmask=pmask,fslice=fslice,
 				rand=rand,**kwargs):
 		yield a
 		
-	    #FrameSequence.pix_iter(self, mask=mask,maxN=maxN,rand=rand,**kwargs)
+	    #FrameSequence.pix_iter(self, mask=mask,max_frames=max_frames,rand=rand,**kwargs)
 
 import PIL.Image as Image
 import matplotlib.image as mpl_img
@@ -609,7 +678,120 @@ class FSeq_multiff(FrameSequence):
                 yield fn(mpl_img.pil_to_array(self.im))
             except EOFError:
                 break
-            
+class FSeq_tiff_2(FSeq_arr):
+    "Class for (multi-frame) tiff files, using tiffile.py by Christoph Gohlke"
+    def __init__(self, fname, ch=None, flipv = False, fliph = False, **kwargs):
+	import tiffile
+	x = tiffile.imread(fname)
+	parent = super(FSeq_tiff_2, self)
+	parent.__init__(x, **kwargs)
+        if ch is not None and self.data.ndim > 3:
+            self.data = self.data[:,:,:,ch]
+        if flipv:
+            self.data = self.data[:,::-1,...]
+        if fliph:
+            self.data = self.data[:,:,::-1,...]
+
+
+
+## -- MES files --
+## TODO: move to a separate file?
+    
+import mesa                      
+
+class FSeq_mes(FSeq_arr):
+    def __init__(self, fname, record=1, ch=None, fns=[],verbose=False):
+        """
+        The following format is assumed:
+        the matlab (.mes) file contains description in a field like "Df0001",
+        and actual data records in fields like 'If0001_001', 'If0001_002'.
+        These fields contain data for the red and green channel, accordingly
+        The images are stored as NXM arrays, where N is one side of an image,
+        and then columns iterate over the other image dimension and time.
+        """
+        self.ch = ch
+        self.fns = fns
+        self.file_name = fname
+        self.record = record
+        self._verbose=verbose
+
+        if type(record) is int:
+            record = 'Df%04d'%record
+        elif type(record) is str:
+            if not ('Df' in record):
+                record = 'Df%04d'%int(record)
+        else:
+            print "Unknown record definition format"
+
+        meta = mesa.load_meta(fname)
+        if verbose:
+            print "file %s has following records:"%fname
+            #print record_keys(meta)
+            for k in mesa.record_keys(meta):
+                if mesa.is_zstack(meta[k]):
+                    desc = 'z_stack'
+                elif mesa.is_xyt(meta[k]):
+                    desc = 'XYT measure'
+                else:
+                    desc = 'unknown'
+                print k, 'is a', desc
+
+        entry = meta[record]
+        if not mesa.is_xyt(entry):
+            print "record %s is not an XYT measurement"%record
+            return
+
+        self._rec_meta = entry
+        self.date = mesa.get_date(entry)
+        self.ffi = mesa.get_ffi(entry)
+
+        nframes, sh = mesa.get_xyt_shape(entry)
+        self._nframes = nframes
+        self._nlines = sh[1]
+        self._shape = sh
+        self.dt = mesa.get_sampling_interval(self.ffi)
+        self.dx = self.dy = 1 #TODO: fix this
+        streams = self.load_record(record)
+        base_shape = (self._nframes-1, self._nlines, self._linesize)
+        if ch is not None:
+            stream = streams[ch]
+            self.data = np.zeros(base_shape, dtype=stream.dtype)
+            for k,f in enumerate(self._reshape_frames(stream)):
+                self.data[k] = f
+        else:
+            streams = [lib.clip_and_rescale(s) for s in streams]
+            reshape_iter = itt.izip(*map(self._reshape_frames, streams))
+            sh = base_shape + (max(3, len(streams)),)
+            self.data = np.zeros(sh, dtype=streams[0].dtype)
+            for k, a in enumerate(reshape_iter):
+                for j, f in enumerate(a):
+                    self.data[k,...,j] = f
+    
+
+    def load_record(self, record):
+        meas_info = mesa.only_measures(self._rec_meta)
+        var_names = [x[0] for x in meas_info['ImageName']]
+        var_names.sort()
+        recs = mesa.io.loadmat(self.file_name, variable_names=var_names)
+        streams = [recs[n] for n in var_names if n in recs]
+        if len(streams) == 0:
+            raise IndexError("can't load record %s"%str(record))
+        self._linesize = streams[0].shape[0]
+        if self._verbose:
+            print 'Number of working channels:', len(streams)
+        return streams
+
+    def _reshape_frames(self, stream):
+        side = self._nlines
+        return (stream[:,k*side:(k+1)*side].T for k in xrange(1,self._nframes))
+      
+
+
+## -- End of MES files --        
+
+
+import inspect
+
 def open_seq(path, *args, **kwargs):
     """Dispatch to an appropriate class constructor depending on the file name
 
@@ -621,25 +803,122 @@ def open_seq(path, *args, **kwargs):
     Returns:
       - `instance`  of an appropriate Frame Sequence class
     """
-    images =  ('bmp', 'jpg', 'jpeg', 'png', 'tif','tiff')
+    images =  ('bmp', 'jpg', 'jpeg', 'png', 'tif','tiff', 'ppm', 'pgm')
     if type(path) is np.ndarray:
 	return FSeq_arr(path, *args, **kwargs)
-    ending = re.findall('[^*\.]+', path)[-1]
+    ending = re.findall('[^*\.]+', path)[-1].lower()
     if ending == 'txt':
-        return FSeq_txt(path, *args, **kwargs)
+        handler = FSeq_txt
+    elif ending == 'mes':
+        handler = FSeq_mes
     elif ending == 'mlf':
-        return FSeq_mlf(path, *args, **kwargs)
+        handler = FSeq_mlf
     elif ending == 'npy':
-        return FSeq_npy(path, *args, **kwargs)
+        handler =  FSeq_npy
     elif ending in images:  # A collection of images or a big tiff
         if '*' in path: # many files
             from imfun import leica
             xml_try = leica.get_xmljob(path.split('*')[0])
             if kwargs.has_key('xmlname') or xml_try:
-                return FSeq_imgleic(path, *args, **kwargs)
+                handler =  FSeq_imgleic
             else:
-                return FSeq_img(path, *args, **kwargs)
+                handler =  FSeq_img
         elif ending in ('tif', 'tiff'): # single multi-frame tiff
-            return FSeq_multiff(path, *args, **kwargs)
-            
+            handler = FSeq_tiff_2
+    spec = inspect.getargspec(handler.__init__)
+    for k in kwargs.keys():
+        if k not in spec[0]:
+            kwargs.pop(k) # skip arguments the __init__ doesn't know about
+    return handler(path, *args, **kwargs)
+
+
+
     
+## HDF5-related stuff
+
+try:
+    import h5py
+
+    class FSeq_hdf5(FSeq_arr):
+        "Base class for hdf5 files"
+        def __init__(self, fname, dataset=None,**kwargs):
+            parent = super(FSeq_hdf5, self)
+            f = h5py.File(fname, 'r')
+
+            if dataset and dataset not in f:
+                print "Dataset name doesn't exist in file, setting to None "
+                dataset is None
+
+            if dataset is None: # no dataset name is provided
+                keys = f.keys()
+                if len(keys) == 1: # there is only one dataset, use it
+                    dataset = keys[0]
+                else:
+                    raise KeyError("No or wrong dataset name provided and the file has\
+                    more than one")
+            arr = f[dataset]
+            parent.__init__(arr,**kwargs)
+
+        def length(self):
+            return self.data.shape[0]
+        def frames(self):
+            fn = self.pipeline()
+            return itt.imap(fn, (f for f in self.data))
+
+
+    class FSeq_hdf5_lsc(FrameSequence):
+        "Class for hdf5 files written by pylsi software"
+        def __init__(self, fname):
+            parent = super(FSeq_hdf5_lsc, self)
+            parent.__init__()
+            f = h5py.File(fname, 'r')
+            t = f['tstamps']
+            self.tv = (t-t[0])/1e6 # relative time, in s
+            self.dt = np.median(np.diff(self.tv))
+            self.data = f['lsc']
+            self.fns = []
+        def length(self):
+            return self.data.shape[0]
+        def frames(self):
+            fn = self.pipeline()
+            return itt.imap(fn, (f for f in self.data))
+        def __getitem__(self, val):
+            x = self.data[val]
+            if self.fns == []:
+                return self.data[val]
+            if type(val) is int:
+                return self.pipeline()(x)
+            out = np.zeros(x.shape,x.dtype)
+            for j,f in enumerate(x):
+                out[j] = self.pipeline()(x)
+            return out
+
+    def fseq2h5(seq, name,compress_level=-1):
+        # todo: add metadata, such as time and spatial scales
+        if os.path.exists(name):
+            sys.stderr.write("File exists, removing\n")
+            os.remove(name)
+        fid = h5py.File(name, 'w')
+        L = seq.length()
+        sh = seq.shape()
+        chunkshape = tuple([1] + list(sh))
+        fullshape = tuple([L] + list(sh))
+        kwargs = dict()
+        if compress_level > -1:
+            kwargs['compression'] = 'gzip'
+            kwargs['compression_opts'] = compress_level
+
+        dset = fid.create_dataset('data', fullshape, dtype=seq[0].dtype,
+                                  chunks = chunkshape, **kwargs)
+        for k,f in enumerate(seq.frames()):
+            dset[k,...] = f
+            sys.stderr.write('\r writing frame %02d out of %03d'%(k,L))
+        fid.close()
+        return name
+
+except ImportError as e:
+    print "Import Error", e
+        
+        
+        
+            
