@@ -59,7 +59,7 @@ def cut_branch(stem, branch):
 
 def max_pos(arr, labels, ind, xslice):
     """Returns position of the maximum value"""
-    offset = np.array([x.start for x in xslice], _dtype_)
+    offset = np.array([x.start for x in xslice], 'int')
     x = ndimage.maximum_position(arr[xslice], labels[xslice], ind)
     return tuple(x + offset)
 
@@ -175,6 +175,38 @@ def restore_object(object, coefs, min_level=0,):
     supp = supp_from_obj(object,min_level)
     return atrous.rec_with_support(coefs, supp)
 
+def _restore_object_iterative(arr, object, min_level=0, niter=5,
+                              dec_fn = atrous.decompose,
+                              coefs = None, fullout = 0,
+                              positive_only = True,
+                              step_size=2,
+                              step_damp=0.85):
+    rws = atrous.rec_with_support
+    # todo: step size damping
+    supp = supp_from_obj(object,min_level)
+    if coefs is None:
+        coefs = dec_fn(arr, nlevels)
+        nlevels = len(supp)
+    else:
+        nlevels = len(coefs)-1
+
+    Xn = rws(coefs,supp)
+    if fullout:
+        out = [Xn]
+        
+    for i in range(niter):
+        upd = rws(coefs - dec_fn(Xn,nlevels),supp)
+        Xnp1 = Xn + step_size*upd
+        step_size *= step_damp
+        if positive_only:
+            Xnp1 *= Xnp1>=0
+        Xn = Xnp1
+        if fullout:
+            out.append(Xn)
+    if not fullout:
+        out = Xn
+    return out
+
 def supp_from_connectivity(graph,nlevels):
     nodes = lib.flatten(graph)
     sh = nodes[0].labels.shape
@@ -236,7 +268,7 @@ def deblend_node_old(node, coefs, acc = None):
             deblend_node(c, coefs, acc)
     return acc
 
-def deblend_node(node, coefs, acc = None, min_scales=2):
+def deblend_node(node, coefs, acc = None):
     """Make an attempt to deblend overlapping objects.
 
     Parameters:
@@ -254,6 +286,7 @@ def deblend_node(node, coefs, acc = None, min_scales=2):
     mxcoef = lambda level, loc : coefs[level][loc]
     sublevel = lambda level: [n for n in flat_leaves if (level-n.level)==1]
     tocut = []
+    ## for each branch we decide if we want to cut it off
     for b in node.branches:
 	wjm = mxcoef(b.level, b.max_pos)
 	if len(b.branches) == 0:
@@ -265,15 +298,21 @@ def deblend_node(node, coefs, acc = None, min_scales=2):
 	wjp1m = np.max(coefs[b.level+1][b.labels==b.ind])
 	## NB! only cut if there are more than one structure at the same level
 	## which belongs to the same tree!
+	#print wjm, wjm1m, wjp1m
 	if (wjm1m < wjm > wjp1m):
+	    #print 'True!'
 	    atlevel = sublevel(node.level)
 	    if len(atlevel)>1:
 		tocut.append(b)
     for c in tocut:
 	free = c.cut()
-	if  nscales(free) >= min_scales:
-	    acc.append(free)
-	    deblend_node(c, coefs, acc)
+        acc.append(free)
+	deblend_node(free, coefs, acc) # this is important to add freely-cut
+                                       # nodes to the acc, not the base (we are
+                                       # already dealing with it)
+    # check if we will need to deblend further down
+    for b in node.branches:
+    	deblend_node(b, coefs, acc)
     return acc
 
 
@@ -345,18 +384,22 @@ def just_denoise(arr, k=3, level=5, noise_std=None,
     new_supp = supp_from_connectivity(g,level)
     return atrous.rec_with_support(coefs, new_supp)
     
-    
+
+import mmt,multiscale    
 
 ### This is one of the main functions ###
 #----------------------------------------
 def find_objects(arr, k=3, level=5, noise_std=None,
                  coefs=None,
                  supp=None,
-		 retraw=False, # only used for testing
+                 dec_fn = atrous.decompose,
+		 retraw=False, # return raw, only used for testing
 		 start_scale=0,
 		 weights=[1., 1., 1., 1., 1.],
+                 deblendp=True,
                  min_px_size=200,
                  min_nscales=2,
+                 rec_variant=2,
 		 modulus = False):
     """Use MVM to find objects in the input array.
 
@@ -388,30 +431,48 @@ def find_objects(arr, k=3, level=5, noise_std=None,
     if np.iterable(k):
         level = len(k)
     if coefs is None:
-        coefs = atrous.decompose(arr, level)
+        coefs = dec_fn(arr, level)
     if noise_std is None:
-	if arr.ndim > 2:
-	    noise_std = atrous.estimate_sigma_mad(coefs[0], True)
-	else:
-	    noise_std = atrous.estimate_sigma(arr, coefs)
+	noise_std =  atrous.estimate_sigma_mad(coefs[0], True)
+	## if arr.ndim > 2:
+	##     noise_std = atrous.estimate_sigma_mad(coefs[0], True)
+	## else:
+	##     noise_std = atrous.estimate_sigma(arr, coefs)
     ## calculate support taking only positive coefficients (light sources)
+    sigmaej = atrous.sigmaej
+    if dec_fn == mmt.decompose_mwt:
+        sigmaej = mmt.sigmaej_mwts2
     if supp is None:
-        supp = atrous.get_support(coefs, np.array(k,_dtype_)*noise_std,
-                                  modulus=modulus)  
+        supp = multiscale.threshold_w(coefs, np.array(k,_dtype_)*noise_std,
+                                      modulus=modulus, sigmaej=sigmaej)  
     structures = get_structures(coefs, supp)
     g = connectivity_graph(structures)
+    if deblendp:
+        gdeblended = deblend_all(g, coefs, min_nscales) # destructive
+    else:
+        gdeblended = [r for r in g if nscales(r) >= min_nscales]
 
-    gdeblended = deblend_all(g, coefs, min_nscales) # destructive
     check = lambda x: len(tree_locations2(x)) > min_px_size
     objects = sorted([x for x in gdeblended if check(x)],
 		     key = lambda u: tree_mass(u), reverse=True)
-    if retraw:
+    if retraw == 1:
 	return objects
-    pipeline = lib.flcompose(lambda x1,x2: supp_from_obj(x1,x2,
-							 weights = weights),
-			     lambda x:atrous.rec_with_support(coefs, x),
-			     embedding)
-    recovered = (pipeline(obj, start_scale) for obj in objects)
+    if retraw == 2:
+        return [supp_from_obj(o,start_scale) for o in objects]
+    # note: even if we decompose with mmt.decompose_mwt
+    # we use atrous.decompose for object reconstruction because
+    # we don't expect too many outliers and this way it's faster
+    pipelines = [lib.flcompose(lambda x1,x2: supp_from_obj(x1,x2,
+                                                           weights = weights),
+                               lambda x: multiscale.simple_rec(coefs, x),
+                               embedding),
+                 lib.flcompose(lambda x1,x2: supp_from_obj(x1,x2,
+                                                           weights = weights),
+                               lambda x:
+                               multiscale.simple_rec_iterative(coefs, x, 
+                                                               positive_only=(not modulus)),
+                               embedding)]
+    recovered = (pipelines[rec_variant-1](obj, start_scale) for obj in objects)
     return filter(lambda x: np.sum(x[0]>0) > min_px_size, recovered)
 # ----------
 
@@ -642,6 +703,8 @@ def objects_to_array(objlist):
 #### -------------------------------------------------####
 
 
+### --- old ----
+
 _phi_ = np.array([1./16, 1./4, 3./8, 1./4, 1./16], _dtype_)
 __x = _phi_.reshape(1,-1)
 _phi2d_ = np.dot(__x.T,__x)
