@@ -31,8 +31,9 @@ from scipy.interpolate import splrep, splev, splprep
 
 
 mpl.rcParams['image.aspect'] = 'auto'
-mpl.rcParams['image.origin'] = 'lower'
 
+#let the user decide where image origin should be
+##mpl.rcParams['image.origin'] = 'lower'
 
 import itertools
 
@@ -109,15 +110,12 @@ def line_reslice2(data, p1, p2):
     return np.array([data[f(p1[1],p2[1])(i), f(p1[0],p2[0])(i)]
                      for i in range(L)])
 
-
 def color_walker():
     ar1 = lib.ar1
     red, green, blue = ar1(), ar1(), ar1()
     while True:
         yield map(lambda x: np.mod(x.next(),1.0), (red,green,blue))
 
-
-
 def rezip(a):
     return zip(*a)
 
@@ -177,7 +175,6 @@ Diameter Manager:
 - type 'a' to calculate propagation velocity
 """
 
-
 class GWExpansionMeasurement1:
     def __init__(self, ax):
 	self.ax = ax
@@ -316,7 +313,7 @@ class GWExpansionMeasurement1:
 	self.canvas.draw()
     def event_ok(self, event):
         return event.inaxes == self.ax and \
-               pl.get_current_fig_manager().toolbar.mode ==''
+               self.canvas.toolbar.mode ==''
 
 
 
@@ -343,7 +340,7 @@ class DraggableObj:
             containsp, _ = self.obj.contains(event)
         return event.inaxes == self.obj.axes and \
                containsp and \
-               pl.get_current_fig_manager().toolbar.mode ==''
+               self.obj.axes.figure.canvas.toolbar.mode ==''
 
     def connect(self):
         "connect all the needed events"
@@ -502,8 +499,6 @@ class LineScan(DraggableObj):
 	elif output == 'function':
 	    return _
 	
-
-
     def show_timeview(self,frange=None,hwidth=2):
         timeview,points = self.get_timeview(frange=frange,hwidth=hwidth)
         if timeview is not None:
@@ -511,11 +506,15 @@ class LineScan(DraggableObj):
 	    f = pl.figure()
             ax = f.add_subplot(111)
             # TODO: work out if sx differs from sy
+            y_extent = (0,fseq.dt*timeview.shape[0])
+            if mpl.rcParams['image.origin'] == 'lower':
+                lowp = 1
+            else:
+                lowp = -1
             ax.imshow(timeview,
-                      extent=(0, self.dx*timeview.shape[1], 0,
-                              fseq.dt*timeview.shape[0]),
-                      )
-                      #aspect='equal')
+                      extent=(0, self.dx*timeview.shape[1]) +\
+                      y_extent[::lowp],
+                      interpolation='nearest')
             if self.scale_setp:
                 ax.set_xlabel('um')
             ax.set_ylabel('time, sec')
@@ -581,8 +580,6 @@ class CircleROI(DraggableObj):
         self.obj.remove()
         self.tagtext.remove()
         self.disconnect()
-        #self.obj.axes.legend()
-        #self.parent.legend()
         self.redraw()
 
     def move(self, p):
@@ -625,8 +622,14 @@ class CircleROI(DraggableObj):
 	  - else, returns :math:`\\Delta v/\\bar v` 
 
 	"""
-        shape = self.parent.fseq.shape()
-        v = self.parent.fseq.mask_reduce(self.in_circle(shape))
+        
+        fullshape = self.parent.fseq.shape()
+        sh = fullshape[:2]
+        v = self.parent.fseq.mask_reduce(self.in_circle(sh))
+        #print len(fullshape), hasattr(self.parent.fseq, 'ch'), self.parent.fseq.ch
+        if len(fullshape)>2 and hasattr(self.parent.fseq, 'ch') \
+           and (self.parent.fseq.ch is not None):
+            v = v[:,self.parent.fseq.ch]
         if normp:
 	    if callable(normp):
 		return normp(v)
@@ -793,7 +796,7 @@ class Picker:
         return self._Nf
     
     def on_click(self,event):
-        tb = pl.get_current_fig_manager().toolbar
+        tb = self.fig.canvas.toolbar
         if event.inaxes != self.ax1 or tb.mode != '': return
 
         x,y = round(event.xdata), round(event.ydata)
@@ -828,13 +831,13 @@ class Picker:
     def event_canvas_ok(self, event):
 	pred = event.inaxes !=self.ax1 or \
 		   self.any_roi_contains(event) or \
-		   pl.get_current_fig_manager().toolbar.mode !=''
+		   self.fig.canvas.toolbar.mode !=''
 	return not pred
     def on_press(self, event):
 	if not self.event_canvas_ok(event) or event.button != 3 :
             return
         self.pressed = event.xdata, event.ydata
-        axrange = self.ax1.get_xbound() + self.ax1.get_ybound()
+        axrange = self.ax1.axis()
         self.curr_line_handle = self.init_line_handle()
         self.ax1.axis(axrange)
         return
@@ -936,20 +939,27 @@ class Picker:
 
         dx,dy, scale_setp = self.fseq.get_scale()
 	sy,sx = self.fseq.shape()[:2]
-        avmin,avmax = self.fseq.data_range()
-        if vmin is None: vmin = avmin
-        if vmax is None: vmax = avmax
-        if hasattr(self.fseq, 'ch'):
-            pl.title("Channel: %s" % ('red', 'green','blue')[self.fseq.ch] )
+        if len(self.fseq.shape()) ==2:
+            if vmin is None or vmax is None:
+                avmin,avmax = self.fseq.data_range()
+            if vmin is None: vmin = avmin
+            if vmax is None: vmax = avmax
+        if hasattr(self.fseq, 'ch') and self.fseq.ch is not None:
+            self.ax1.set_title("Channel: %s" % ('red', 'green','blue')[self.fseq.ch] )
+
         if type(mean_frame) is np.ndarray:
 	    f = mean_frame
 	elif mean_frame:
-            f = self.fseq.mean_frame()
+            dtype = self.fseq[0].dtype
+            f = self.fseq.mean_frame().astype(dtype)
+            if np.ndim(f) > 2 and dtype != 'uint8' and f.max() > 1:
+                f = lib.rescale(f)
         else:
             f = self.fseq.frames().next()
 	self.mean_frame = f
+        lowp = [1,-1][mpl.rcParams['image.origin']=='upper']
         self.pl = self.ax1.imshow(f,
-                                  extent = (0, sx*dx, 0, sy*dy),
+                                  extent = (0, sx*dx)+(0, sy*dy)[::lowp],
                                   interpolation = interpolation,
                                   aspect='equal',
                                   vmax=vmax,  vmin=vmin,
@@ -960,12 +970,23 @@ class Picker:
 
         self.disconnect()
         self.connect()
-	#self.fig.show()
+	self.fig.canvas.draw()
         return self.ax1, self.pl, self
+
+    def set_frame_index(self,n):
+        Nf = self.fseq.length()
+	fi = n%Nf
+        self.frame_index = fi
+        _title = '%03d (%3.3f sec)'%(fi, fi*self.fseq.dt)
+        show_f = self.fseq[fi]
+        self.pl.set_data(show_f)
+        self.ax1.set_title(_title)
+        self.fig.canvas.draw()
+
+        
     def frame_skip(self,event, n=1):
 	if not self.event_canvas_ok(event):
 	    return
-	Nf = self.fseq.length()
 	fi = self.frame_index
 	key = hasattr(event, 'button') and event.button or event.key
 	prev_keys = [4,'4','down','left','p']
@@ -976,18 +997,18 @@ class Picker:
 	    if key in home_keys:
 		show_f = self.mean_frame
 		_title = ""
+                self.pl.set_data(show_f)
+                self.ax1.set_title(_title)
+                self.fig.canvas.draw()
+
 	    else:
 		if key in prev_keys:
 		    fi -= n
 		elif key in next_keys:
 		    fi += n
-		fi = fi%Nf
-		self.frame_index = fi
-		_title = '%03d (%3.3f sec)'%(fi, fi*self.fseq.dt)
-		show_f = self.fseq[fi]
-	    self.pl.set_data(show_f)
-	    self.ax1.set_title(_title)
-	    self.fig.canvas.draw()
+		self.set_frame_index(fi)
+        if hasattr(self, 'caller'): #called from frame_viewer
+            self.caller.frame_index = self.frame_index
 
     def connect(self):
         "connect all the needed events"
@@ -1044,7 +1065,14 @@ class Picker:
         t = self.timevec()
         for x,tag,roi,ax in self.roi_show_iterator(rois, **keywords):
 	    if self.isCircleROI(tag):
-		ax.plot(t, x, color = roi.get_facecolor())
+                if np.ndim(x) == 1:
+                    ax.plot(t, x, color = roi.get_facecolor(), label=tag)
+                else:
+                    colors = ('r','g','b')
+                    for k,xe in enumerate(x.T):
+                        color = colors[k%3]
+                        if np.any(xe):
+                            ax.plot(t, xe, color=color,label=tag+':'+color)
 	    else:
 		sh = x.shape
 		ax.imshow(x.T, extent=(t[0],t[-1],0,sh[1]*self.fseq.dx),
@@ -1101,6 +1129,7 @@ class Picker:
                                  vmax = None,
                                  normp = True,
                                  **keywords):
+	from swan import utils as swu
         "Create a figure of a signal, spectrogram and a colorbar"
         if not self.isCircleROI(roitag):
             print "This is not a circle ROI, exiting"
@@ -1114,7 +1143,7 @@ class Picker:
         L = min(Ns,len(tvec))
         tvec,signal = tvec[:L],signal[:L]
         lc = self.roi_objs[roitag].get_color()
-        fig,axlist = lib.setup_axes_for_spectrogram((8,4))
+        fig,axlist = swu.setup_axes_for_spectrogram((8,4))
         axlist[1].plot(tvec, signal,'-',color=lc)
         axlist[1].set_xlabel('time, s')
         utils.wavelet_specgram(signal, f_s, freqs,  axlist[0], vmax=vmax,
@@ -1152,21 +1181,24 @@ class Picker:
 	
 	L = len(rois)
 	if L < 1: return
-	fig = pl.figure(figsize=(8,4.5), dpi=80)
+	fig,axlist = pl.subplots(len(rois),1,sharey=True,sharex=True,
+                                 squeeze = False,
+                                 figsize=(5*L,4.5))
+        
 	for i, roi_label in enumerate(rois):
 	    roi = self.roi_objs[roi_label]
-	    if i == 0:
-		ax = fig.add_subplot(L,1,i+1)
-		ax0 = ax
-	    else:
-		ax = fig.add_subplot(L,1,i+1, sharex = ax0)
+            ax = axlist[i,0]
 	    if self.isCircleROI(roi_label):
 		x = roi.get_timeview(**kwargs)
 	    else:
 		x,points = roi.get_timeview(**kwargs)
 	    if i == L-1:
 		ax.set_xlabel("time, sec")
-	    ax.set_ylabel(roi_label)
+            if L == 1:
+                ax.set_title(roi_label, color=roi.get_color(),
+                             backgroundcolor='w', size='large')
+            else:
+                ax.set_ylabel(roi_label, color=roi.get_color(),size='large')
 	    yield x, roi_label, roi.obj, ax
 
 
